@@ -15,11 +15,11 @@ import (
 func init() {
 	log.SetLevel("trace")
 
-	go tcp.Run("debug", "8081", "pass123", "8082,8083,8084,8085")
-	go tcp.Run("debug", "8082", "pass123")
-	go tcp.Run("debug", "8083", "pass123")
-	go tcp.Run("debug", "8084", "pass123")
-	go tcp.Run("debug", "8085", "pass123")
+	go tcp.Run("debug", "localhost", "8081", "pass123", "8082,8083,8084,8085")
+	go tcp.Run("debug", "localhost", "8082", "pass123")
+	go tcp.Run("debug", "localhost", "8083", "pass123")
+	go tcp.Run("debug", "localhost", "8084", "pass123")
+	go tcp.Run("debug", "localhost", "8085", "pass123")
 	time.Sleep(1 * time.Second)
 }
 
